import unittest
from unittest.mock import Mock, patch

import numpy as np

from pyqtgraph import PlotCurveItem, ScatterPlotItem

from AnyQt.QtCore import Qt, QPoint
from AnyQt.QtTest import QTest
from PyQt5.QtWidgets import QToolTip

from Orange.data import Table, StringVariable, Domain, ContinuousVariable
from Orange.data.filter import Values, FilterString
from Orange.widgets.tests.base import WidgetTest
from Orange.widgets.widget import OWWidget

from orangecontrib.bioinformatics.utils import serverfiles

from orangecontrib.single_cell.widgets.owdropout import OWDropout, \
    DropoutGraph, FilterType


class DummyWidget(OWWidget):
    name = "dummy"


class DummyPoint:
    xi = iter(range(10))
    yi = iter(range(10))

    def x(self):
        return next(self.xi)

    def y(self):
        return next(self.yi) * 2


class TestDropoutGraph(WidgetTest):
    def setUp(self):
        self.parent = DummyWidget()
        self.graph = DropoutGraph(self.parent)
        self.results = results = Mock()
        results.decay = 1
        results.x_offset = 0.1
        results.y_offset = 0.1
        results.mean_expr = np.array([0.1, 0.2])
        results.zero_rate = np.array([0.1, 0.2])
        results.threshold = 0
        self.data = Table(Domain([ContinuousVariable("A"),
                                  ContinuousVariable("B")]),
                          np.array([[1, 0], [0, 0], [2, 0]]))
        self.genes = Table(Domain([], metas=[StringVariable("Entrez ID")]),
                           np.empty((1, 0)), metas=np.array([["1"]]))

    def test_set_data(self):
        self.graph.set_data(self.results, self.data, None)
        self.assertEqual(len(self.graph.plotItem.items), 2)

        self.graph.update_markers(None, self.genes)
        self.assertEqual(len(self.graph.plotItem.items), 2)

        self.graph.update_markers(self.data, self.genes)
        self.assertEqual(len(self.graph.plotItem.items), 3)

        self.graph.update_markers(self.data, None)
        self.assertEqual(len(self.graph.plotItem.items), 2)

    def test_set_data_and_markers(self):
        self.graph.set_data(self.results, self.data, self.genes)
        self.assertEqual(len(self.graph.plotItem.items), 3)

        self.graph.update_markers(self.data, None)
        self.assertEqual(len(self.graph.plotItem.items), 2)

        self.graph.update_markers(self.data, self.genes)
        self.assertEqual(len(self.graph.plotItem.items), 3)

    def test_update_curve(self):
        self.graph.update_curve(self.results)
        self.assertEqual(len(self.graph.plotItem.items), 1)

        self.graph.set_data(self.results, self.data, None)
        self.assertEqual(len(self.graph.plotItem.items), 2)

        self.graph.update_curve(self.results)
        self.assertEqual(len(self.graph.plotItem.items), 2)

    def test_update_markers(self):
        self.graph.update_markers(self.data, self.genes)
        self.assertEqual(len(self.graph.plotItem.items), 0)

        self.graph.update_markers(self.data, None)
        self.assertEqual(len(self.graph.plotItem.items), 0)

        self.graph.update_markers(None, self.genes)
        self.assertEqual(len(self.graph.plotItem.items), 0)

        self.graph.update_markers(None, None)
        self.assertEqual(len(self.graph.plotItem.items), 0)

    def test_on_curve(self):
        self.assertFalse(self.graph._on_curve(1, 0.5))
        self.graph.set_data(self.results, self.data, None)
        self.assertTrue(self.graph._on_curve(1, 0.5))
        self.assertFalse(self.graph._on_curve(1, 1))

    @patch("orangecontrib.single_cell.widgets.owdropout.DropoutGraph."
           "_on_curve", Mock(return_value=True))
    @patch.object(PlotCurveItem, "mapFromScene",
                  Mock(return_value=DummyPoint()))
    def test_cursor_event(self):
        graph = self.graph
        slot = Mock()
        graph.curve_moved.connect(slot)
        graph.set_data(self.results, self.data, None)
        graph.cursor_event(Mock())
        QTest.mousePress(graph.viewport(), Qt.LeftButton, pos=QPoint(1, 1))
        # QTest.mouseMove(g.viewport()) does not work; set flag manually
        graph._state = 3
        graph.cursor_event(Mock())
        slot.assert_called_once_with(1, 2)

    @patch("orangecontrib.single_cell.widgets.owdropout.DropoutGraph._dotAt")
    @patch.object(ScatterPlotItem, "mapFromScene",
                  Mock(return_value=DummyPoint()))
    @patch.object(QToolTip, "showText")
    def test_help_event(self, show_text, dot_at):
        graph = self.graph
        graph.set_data(self.results, self.data, None)
        dots = graph.plotItem.items[0]
        dot_at.return_value = dots.points()[0]
        graph.help_event(Mock())
        self.assertEqual(show_text.call_args[0][1],
                         "A\nExpressed in 2/3 cells (66.7%)")


class TestOWDropout(WidgetTest):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.data = Table("https://datasets.biolab.si/sc/aml-1k.tab.gz")
        genes_path = serverfiles.localpath_download("marker_genes",
                                                    "panglao_gene_markers.tab")
        filter_ = FilterString("Organism", FilterString.Equal, "Human")
        cls.genes = Values([filter_])(Table(genes_path))
        cls.iris = Table("iris")

    def setUp(self):
        self.widget = self.create_widget(OWDropout)

    def test_input_data(self):
        self.send_signal(self.widget.Inputs.data, self.data)
        self.send_signal(self.widget.Inputs.data, self.iris)
        self.send_signal(self.widget.Inputs.data, self.genes)
        self.send_signal(self.widget.Inputs.data, None)

    @patch("orangecontrib.single_cell.widgets.owdropout."
           "DropoutGraph.update_markers")
    @patch("orangecontrib.single_cell.widgets.owdropout."
           "DropoutGraph.set_data")
    def test_input_genes(self, set_data: Mock, update_markers: Mock):
        # genes
        self.send_signal(self.widget.Inputs.genes, self.genes)
        update_markers.assert_called_once_with(None, self.genes)
        update_markers.reset_mock()
        set_data.assert_not_called()

        # data, genes
        self.send_signal(self.widget.Inputs.data, self.data)
        update_markers.assert_not_called()
        update_markers.reset_mock()
        set_data.assert_called_once()

        # data
        self.send_signal(self.widget.Inputs.genes, None)
        update_markers.assert_called_once_with(self.data, None)
        update_markers.reset_mock()
        set_data.assert_called_once()

        # data, genes
        self.send_signal(self.widget.Inputs.genes, self.iris)
        self.assertTrue(self.widget.Warning.missing_entrez_id.is_shown())
        update_markers.assert_called_once_with(self.data, None)
        update_markers.reset_mock()
        set_data.assert_called_once()

        # data
        self.send_signal(self.widget.Inputs.genes, None)
        self.assertFalse(self.widget.Warning.missing_entrez_id.is_shown())
        update_markers.assert_called_once_with(self.data, None)
        set_data.assert_called_once()

    def test_controls_enabled(self):
        controls = self.widget.controls
        buttons = controls.filter_type.buttons
        self.assertEqual(self.widget.filter_type, FilterType.ByNumber)
        self.assertTrue(controls.n_genes.isEnabled())
        self.assertFalse(controls.decay.isEnabled())
        self.assertFalse(controls.x_offset.isEnabled())
        self.assertFalse(controls.y_offset.isEnabled())

        buttons[1].click()
        self.assertEqual(self.widget.filter_type, FilterType.ByEquation)
        self.assertFalse(controls.n_genes.isEnabled())
        self.assertTrue(controls.decay.isEnabled())
        self.assertTrue(controls.x_offset.isEnabled())
        self.assertTrue(controls.y_offset.isEnabled())

    @patch("orangecontrib.single_cell.widgets.owdropout."
           "DropoutGraph.update_curve")
    @patch("orangecontrib.single_cell.widgets.owdropout."
           "DropoutGraph.set_data")
    def test_manual_move(self, set_data: Mock, update_curve: Mock):
        self.send_signal(self.widget.Inputs.data, self.data)
        set_data.assert_called_once()
        update_curve.assert_not_called()
        self.widget.graph.curve_moved.emit(1, 2)
        set_data.assert_called_once()
        update_curve.assert_called_once()
        self.assertEqual(self.widget.filter_type, FilterType.ByEquation)
        controls = self.widget.controls
        self.assertTrue(controls.filter_type.buttons[1].clicked)
        self.assertFalse(controls.n_genes.isEnabled())
        self.assertTrue(controls.decay.isEnabled())
        self.assertTrue(controls.x_offset.isEnabled())
        self.assertTrue(controls.y_offset.isEnabled())

    def test_output(self):
        self.send_signal(self.widget.Inputs.data, self.data)
        output = self.get_output(self.widget.Outputs.data)
        self.assertEqual(len(output.domain.attributes), 497)
        self.assertTrue(self.widget.Warning.less_selected.is_shown())
        self.send_signal(self.widget.Inputs.data, None)
        self.assertIsNone(self.get_output(self.widget.Outputs.data))
        self.assertFalse(self.widget.Warning.less_selected.is_shown())

    def test_info(self):
        self.assertEqual(self.widget.info_label.text(), "No data on input.")
        self.send_signal(self.widget.Inputs.data, self.data)
        text = "Data with 1000 cells and 1000 genes\n497 genes in selection"
        self.assertEqual(self.widget.info_label.text(), text)
        self.send_signal(self.widget.Inputs.data, self.data[:1])
        text = "Data with 1 cell and 1000 genes\n0 genes in selection"
        self.assertEqual(self.widget.info_label.text(), text)
        self.send_signal(self.widget.Inputs.data, None)
        self.assertEqual(self.widget.info_label.text(), "No data on input.")

    def test_param_changed(self):
        self.widget.controls.n_genes.setValue(100)
        self.send_signal(self.widget.Inputs.data, self.data)
        self.widget.controls.n_genes.setValue(200)
        output = self.get_output(self.widget.Outputs.data)
        self.assertEqual(len(output.domain.attributes), 200)
        self.send_signal(self.widget.Inputs.data, None)
        self.widget.controls.n_genes.setValue(100)


if __name__ == "__main__":
    unittest.main()



import os
import sys
import Orange.widgets.data.owdatasets

from AnyQt.QtWidgets import QApplication
from AnyQt.QtGui import QStandardItemModel, QStandardItem
from AnyQt.QtCore import Qt

from Orange.widgets.gui import IndicatorItemDelegate

from orangecontrib.bioinformatics.ncbi.taxonomy import shortname, common_taxids
from orangecontrib.bioinformatics.ncbi.gene import ENTREZ_ID
from orangecontrib.bioinformatics.widgets.utils.data import GENE_AS_ATTRIBUTE_NAME, TAX_ID, GENE_ID_ATTRIBUTE


class OWscDataSets(Orange.widgets.data.owdatasets.OWDataSets):
    name = "Single Cell Datasets"
    description = "Load a data set from an online repository"
    icon = "icons/SingleCellDatasets.svg"
    priority = 150

    INDEX_URL = "https://datasets.biolab.si/sc/"
    DATASET_DIR = "sc-datasets"

    HEADER_SCHEMA = [
        ['islocal',      {'label': ''}],
        ['title',        {'label': 'Title'}],
        ['size',         {'label': 'Size'}],
        ['instances',    {'label': 'Cells'}],
        ['variables',    {'label': 'Genes'}],
        ['taxid',        {'label': 'Organism'}],
        ['target',       {'label': 'Target'}],
        ['tags',         {'label': 'Tags'}]
    ]

    def load_data(self, path):
        info = self.selected_dataset()
        data = Orange.data.Table(path)
        data.attributes[TAX_ID] = info.taxid
        data.attributes[GENE_AS_ATTRIBUTE_NAME] = True  # Will all data sets have gene names in columns?
        data.attributes[GENE_ID_ATTRIBUTE] = ENTREZ_ID
        return data

    def assign_delegates(self):
        self.view.setItemDelegateForColumn(
            self.Header.islocal, IndicatorItemDelegate(self, role=Qt.DisplayRole)
        )
        self.view.setItemDelegateForColumn(
            self.Header.size,
            Orange.widgets.data.owdatasets.SizeDelegate(self))

        self.view.setItemDelegateForColumn(
            self.Header.instances,
            Orange.widgets.data.owdatasets.NumericalDelegate(self)
        )
        self.view.setItemDelegateForColumn(
            self.Header.variables,
            Orange.widgets.data.owdatasets.NumericalDelegate(self)
        )

    def create_model(self):
        allkeys = set(self.allinfo_local)

        if self.allinfo_remote is not None:
            allkeys = allkeys | set(self.allinfo_remote)

        allkeys = sorted(allkeys)

        model = QStandardItemModel(self)
        model.setHorizontalHeaderLabels(self._header_labels)

        current_index = -1
        for i, file_path in enumerate(allkeys):
            data_info = self._parse_info(file_path)
            row = []

            for info_tag, header_setting in self.HEADER_SCHEMA:
                item = QStandardItem()

                try:
                    data = data_info.__getattribute__(info_tag)
                except AttributeError:
                    # unknown tag in JSON
                    data = ''

                # first column indicating cached data sets
                if info_tag == 'islocal':
                    item.setData(' ' if data else '', Qt.DisplayRole)
                    item.setData(data_info, Qt.UserRole)

                else:
                    # parse taxid to common name
                    if info_tag == 'taxid' and data in common_taxids():
                        data = shortname(data)[0].title()

                    if info_tag == 'tags':
                        data = ', '.join(data) if data else ''

                    item.setData(data, Qt.DisplayRole)

                # set icon to Target column
                if info_tag == 'target' and data:
                    item.setIcon(Orange.widgets.data.owdatasets.variable_icon(data))

                row.append(item)
            model.appendRow(row)

            if os.path.join(*file_path) == self.selected_id:
                current_index = i

        return model, current_index


def main(args=None):
    if args is None:
        args = sys.argv

    app = QApplication(list(args))
    w = OWscDataSets()
    w.show()
    w.raise_()
    rv = app.exec_()
    w.saveSettings()
    w.onDeleteWidget()
    return rv


if __name__ == "__main__":
    sys.exit(main())




import os
import sys

from typing import List

from serverfiles import sizeformat

from AnyQt.QtCore import Qt, QFileInfo, QTimer, Signal
from AnyQt.QtGui import QStandardItemModel, QStandardItem
from AnyQt.QtWidgets import (
    QSizePolicy, QGridLayout, QHBoxLayout, QFormLayout,
    QLabel, QComboBox, QSpinBox, QCheckBox, QPushButton,
    QStyle, QApplication, QFileDialog, QFileIconProvider,
    QWidget
)
from AnyQt.QtCore import pyqtSlot as Slot

from Orange.data import Table
from Orange.widgets import widget, gui, settings
from Orange.widgets.utils.filedialogs import RecentPath
from Orange.widgets.utils.buttons import VariableTextPushButton

from orangecontrib.single_cell.widgets.load_data import get_data_loader, Loader


Formats = [
    "Count file (*.count)",
    "Tab separated file (*.tsv *.tab)",
    "Comma separated file (*.csv)",
    "10x gene-barcode matrix (matrix.mtx)",
    "Microsoft Excel spreadsheet (*.xls *.xlsx)",
    "Pickled Python object file (*.pkl *.pickle)",
    "All files (*.*)"
]

AnnotationFormats = [
    "Meta file (*.meta)",
    "Tab separated file (*.tsv *.tab)",
    "Comma separated file (*.csv)",
    "All files (*.*)",
]


def RecentPath_asqstandarditem(pathitem):
    # type: (RecentPath) -> QStandardItem
    icon_provider = QFileIconProvider()
    # basename of a normalized name (strip right path component separators)
    basename = os.path.basename(os.path.normpath(pathitem.abspath))
    item = QStandardItem(
        icon_provider.icon(QFileInfo(pathitem.abspath)),
        basename
    )
    item.setToolTip(pathitem.abspath)
    item.setData(pathitem, Qt.UserRole)
    return item


def init_recent_paths_model(model, paths, relpaths=[]):
    # type: (QStandardItemModel, List[RecentPath]) -> None
    for pathitem in paths:
        item = RecentPath_asqstandarditem(pathitem)
        abspath = pathitem.search(searchpaths=relpaths)
        if not (abspath and os.path.exists(abspath)):
            item.setEnabled(False)
            item.setSelectable(False)
            item.setToolTip(item.toolTip() + " (Missing from file system)")
        model.appendRow(item)


def insert_recent_path(model, path, relpaths=[]):
    # type: (QStandardItemModel, RecentPath) -> int
    index = -1
    for i in range(model.rowCount()):
        item = model.item(i, 0)
        pathitem = item.data(Qt.UserRole)
        if isinstance(pathitem, RecentPath) and \
                samepath(pathitem.abspath, path.abspath):
            index = i
            break
    if index != -1:
        item = model.takeRow(index)
    else:
        item = RecentPath_asqstandarditem(path)
    model.insertRow(0, item)
    return 0


def samepath(p1, p2):
    return (os.path.normpath(os.path.normcase(p1)) ==
            os.path.normpath(os.path.normcase(p2)))


class RunaroundSettingsHandler(settings.SettingsHandler):
    def pack_data(self, widget):
        widget._saveState()
        return super().pack_data(widget)


class OWLoadData(widget.OWWidget):
    name = ""
    icon = "icons/LoadData.svg"
    priority = 110

    class Outputs:
        data = widget.Output("Data", Table)

    class Information(widget.OWWidget.Information):
        modified = widget.Msg(
            "Uncommited changes\nPress 'Load data' to submit changes"
        )

    class Warning(widget.OWWidget.Warning):
        sampling_in_effect = widget.Msg("Sampling is in effect.")

    class Error(widget.OWWidget.Error):
        row_annotation_mismatch = widget.Msg(
            "Row annotation length mismatch\n"
            "Expected {} rows got {}"
        )
        col_annotation_mismatch = widget.Msg(
            "Column annotation length mismatch\n"
            "Expected {} rows got {}"
        )
        inadequate_headers = widget.Msg(
            "Headers and Row Labels error"
        )
        reading_error = widget.Msg(
            "Cannot read data using given parameters."
        )

    _recent = settings.Setting([])  # type: List[RecentPath]
    _recent_row_annotations = settings.Setting([])  # type: List[RecentPath]
    _recent_col_annotations = settings.Setting([])  # type: List[RecentPath]
    _cells_in_rows = settings.Setting(False)
    _col_annotations_enabled = settings.Setting(False)
    _row_annotations_enabled = settings.Setting(False)

    _last_path = settings.Setting("")  # type: str
    _header_rows_count = settings.Setting(1)  # type: int
    _header_cols_count = settings.Setting(1)  # type: int
    _sample_rows_enabled = settings.Setting(False)  # type: bool
    _sample_cols_enabled = settings.Setting(False)  # type: bool
    _sample_cols_p = settings.Setting(10.0)  # type: bool
    _sample_rows_p = settings.Setting(10.0)  # type: bool

    settingsHandler = RunaroundSettingsHandler()

    want_main_area = False
    resizing_enabled = False

    cells_in_rows_changed = Signal()

    def __init__(self):
        super().__init__()
        self._current_path = ""
        self._data_loader = Loader()
        icon_open_dir = self.style().standardIcon(QStyle.SP_DirOpenIcon)

        # Top grid with file selection combo box
        self.file_layout = grid = QGridLayout()
        lb = QLabel("File:")
        lb.setSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
        self.recent_combo = cb = QComboBox(
            sizeAdjustPolicy=QComboBox.AdjustToMinimumContentsLengthWithIcon,
            minimumContentsLength=20,
            toolTip="Select a recent file"
        )
        self.recent_model = cb.model()  # type: QStandardItemModel
        self.recent_combo.activated[int].connect(self._select_recent)

        browse = QPushButton("...", autoDefault=False, icon=icon_open_dir,
                             clicked=self.browse)

        # reload = QPushButton("Reload", autoDefault=False, icon=icon_reload)

        grid.addWidget(lb, 0, 0, Qt.AlignVCenter)
        grid.addWidget(cb, 0, 1)
        grid.addWidget(browse, 0, 2)
        # grid.addWidget(reload, 0, 3)

        self.summary_label = label = QLabel("", self)
        label.ensurePolished()
        f = label.font()
        if f.pointSizeF() != -1:
            f.setPointSizeF(f.pointSizeF() * 5 / 6)
        else:
            f.setPixelSize(f.pixelSize() * 5 / 6)
        label.setFont(f)
        grid.addWidget(label, 1, 1, 1, 3)

        self.controlArea.layout().addLayout(grid)

        box = gui.widgetBox(
            self.controlArea, "Headers and Row Labels", spacing=-1
        )
        hl = QHBoxLayout()
        hl.setContentsMargins(0, 0, 0, 0)
        self.header_rows_spin = spin = QSpinBox(
            box, minimum=0, maximum=3, value=self._header_rows_count,
            keyboardTracking=False
        )
        spin.valueChanged.connect(self.set_header_rows_count)
        hl.addWidget(QLabel("Data starts with", box))
        hl.addWidget(self.header_rows_spin)
        hl.addWidget(QLabel("header row(s)", box))
        hl.addStretch(10)
        box.layout().addLayout(hl)

        hl = QHBoxLayout()
        hl.setContentsMargins(0, 0, 0, 0)
        self.header_cols_spin = spin = QSpinBox(
            box, minimum=0, maximum=3, value=self._header_cols_count,
            keyboardTracking=False
        )
        spin.valueChanged.connect(self.set_header_cols_count)

        hl.addWidget(QLabel("First", box))
        hl.addWidget(self.header_cols_spin)
        hl.addWidget(QLabel("column(s) are row labels", box))
        hl.addStretch(10)
        box.layout().addLayout(hl)

        self.data_struct_box = box = gui.widgetBox(
            self.controlArea, "Input Data Structure"
        )
        gui.radioButtons(
            box, self, "_cells_in_rows",
            ["Genes in rows, cells in columns",
             "Cells in rows, genes in columns"],
            callback=self._cells_in_rows_changed
        )

        box = gui.widgetBox(
            self.controlArea, "Sample Data", spacing=-1)

        grid = QGridLayout()
        grid.setContentsMargins(0, 0, 0, 0)
        box.layout().addLayout(grid)

        self.sample_rows_cb = cb = QCheckBox(checked=self._sample_rows_enabled)
        self.sample_rows_p_spin = spin = QSpinBox(
            minimum=0, maximum=100, value=self._sample_rows_p
        )
        spin.valueChanged.connect(self.set_sample_rows_p)
        suffix = QLabel("% of cells")
        cb.toggled.connect(self.set_sample_rows_enabled)

        grid.addWidget(cb, 0, 0)
        grid.addWidget(spin, 0, 1)
        grid.addWidget(suffix, 0, 2)

        self.sample_cols_cb = cb = QCheckBox(checked=self._sample_cols_enabled)
        self.sample_cols_p_spin = spin = QSpinBox(
            minimum=0, maximum=100, value=self._sample_cols_p
        )
        spin.valueChanged.connect(self.set_sample_cols_p)
        suffix = QLabel("% of genes")
        cb.toggled.connect(self.set_sample_cols_enabled)

        grid.addWidget(cb, 1, 0)
        grid.addWidget(spin, 1, 1)
        grid.addWidget(suffix, 1, 2)
        grid.setColumnStretch(3, 10)

        self.annotation_files_box = box = gui.widgetBox(
            self.controlArea, "Cell && Gene Annotation Files"
        )
        form = QFormLayout(
            formAlignment=Qt.AlignLeft,
            rowWrapPolicy=QFormLayout.WrapAllRows,
        )
        box.layout().addLayout(form)

        self.row_annotations_cb = cb = QCheckBox(
            "Cell annotations", checked=self._row_annotations_enabled
        )
        self._row_annotations_w = w = QWidget(enabled=self._row_annotations_enabled)
        cb.toggled.connect(self.set_row_annotations_enabled)
        cb.toggled.connect(w.setEnabled)
        hl = QHBoxLayout()
        hl.setContentsMargins(0, 0, 0, 0)
        w.setLayout(hl)
        self.row_annotations_combo = QComboBox(
            sizeAdjustPolicy=QComboBox.AdjustToMinimumContentsLengthWithIcon,
            minimumContentsLength=18
        )
        self.row_annotations_combo.activated.connect(
            self._row_annotations_combo_changed
        )
        hl.addWidget(self.row_annotations_combo)
        hl.addWidget(QPushButton("...", box, autoDefault=False,
                                 icon=icon_open_dir,
                                 clicked=self.browse_row_annotations))
        # hl.addWidget(QPushButton("Reload", box, autoDefault=False,
        #                          icon=icon_reload))
        form.addRow(cb, w)

        self.col_annotations_cb = cb = QCheckBox(
            "Gene annotations", checked=self._col_annotations_enabled
        )
        self._col_annotations_w = w = QWidget(enabled=self._col_annotations_enabled)
        cb.toggled.connect(self.set_col_annotations_enabled)
        cb.toggled.connect(w.setEnabled)
        hl = QHBoxLayout()
        hl.setContentsMargins(0, 0, 0, 0)
        w.setLayout(hl)
        self.col_annotations_combo = QComboBox(
            sizeAdjustPolicy=QComboBox.AdjustToMinimumContentsLengthWithIcon,
            minimumContentsLength=18
        )
        self.col_annotations_combo.activated.connect(
            self._col_annotations_combo_changed
        )
        hl.addWidget(self.col_annotations_combo)
        hl.addWidget(QPushButton("...", box, autoDefault=False,
                                 icon=icon_open_dir,
                                 clicked=self.browse_col_annotations))
        # hl.addWidget(QPushButton("Reload", box, autoDefault=False,
        #                          icon=icon_reload))
        form.addRow(cb, w)

        self.controlArea.layout().addStretch(10)
        self.load_data_button = button = VariableTextPushButton(
            "Load data", autoDefault=True, textChoiceList=["Load data", "Reload"]
        )
        self.load_data_button.setAutoDefault(True)
        button.clicked.connect(self.commit, Qt.QueuedConnection)
        self.controlArea.layout().addWidget(button, alignment=Qt.AlignRight)

        init_recent_paths_model(
            self.recent_model,
            [self.resolve_path(p) for p in self._recent],
        )
        init_recent_paths_model(
            self.row_annotations_combo.model(),
            [self.resolve_path(p) for p in self._recent_row_annotations]
        )
        init_recent_paths_model(
            self.col_annotations_combo.model(),
            [self.resolve_path(p) for p in self._recent_col_annotations]
        )
        self.__update_summary()
        self._update_warning()

        if self._last_path != "" and os.path.exists(self._last_path):
            QTimer.singleShot(
                0, lambda: self.set_current_path(self._last_path)
            )
        else:
            self.recent_combo.setCurrentIndex(-1)

    def resolve_path(self, path):
        basedir = self.workflowEnv().get("basedir", None)
        if not basedir or not path:
            return path
        return path.resolve([("basedir", basedir)]) or path

    def _cells_in_rows_changed(self):
        self._data_loader.transposed = not self._cells_in_rows
        self._invalidate()
        self.cells_in_rows_changed.emit()

    def _row_annotations_combo_changed(self):
        path = self.row_annotations_combo.currentData(Qt.UserRole)
        if isinstance(path, RecentPath) and os.path.exists(path.abspath):
            self._data_loader.row_annotation_file = path  # type: RecentPath
        else:
            self._data_loader.row_annotation_file = None
        self._invalidate()

    def _col_annotations_combo_changed(self):
        path = self.col_annotations_combo.currentData(Qt.UserRole)
        if isinstance(path, RecentPath) and os.path.exists(path.abspath):
            self._data_loader.col_annotation_file = path  # type: RecentPath
        else:
            self._data_loader.col_annotation_file = None
        self._invalidate()

    def _update_warning(self):
        if (self._sample_rows_enabled and self._sample_rows_p < 100) or \
                (self._sample_cols_enabled and self._sample_cols_p < 100):
            self.Warning.sampling_in_effect()
        else:
            self.Warning.sampling_in_effect.clear()

    def set_sample_rows_enabled(self, enabled, commit=True):
        if self._sample_rows_enabled != enabled:
            self._sample_rows_enabled = enabled
            self.sample_rows_cb.setChecked(enabled)
            self._update_warning()
            self._data_loader.sample_rows_enabled = enabled
            if commit:
                self._invalidate()

    def set_sample_cols_enabled(self, enabled, commit=True):
        if self._sample_cols_enabled != enabled:
            self._sample_cols_enabled = enabled
            self.sample_cols_cb.setChecked(enabled)
            self._update_warning()
            self._data_loader.sample_cols_enabled = enabled
            if commit:
                self._invalidate()

    def set_sample_rows_p(self, p, commit=True):
        if self._sample_rows_p != p:
            self._sample_rows_p = p
            self._update_warning()
            self.sample_rows_p_spin.setValue(p)
            self._data_loader.sample_rows_p = p
            if commit:
                self._invalidate()

    def set_sample_cols_p(self, p, commit=True):
        if self._sample_cols_p != p:
            self._sample_cols_p = p
            self._update_warning()
            self.sample_cols_p_spin.setValue(p)
            self._data_loader.sample_cols_p = p
            if commit:
                self._invalidate()

    def set_header_rows_count(self, n, commit=True):
        if self._header_rows_count != n:
            self._header_rows_count = n
            self.header_rows_spin.setValue(n)
            self._data_loader.header_rows_count = n
            if commit:
                self._invalidate()

    def set_header_cols_count(self, n, commit=True):
        if self._header_cols_count != n:
            self._header_cols_count = n
            self.header_cols_spin.setValue(n)
            self._data_loader.header_cols_count = n
            if commit:
                self._invalidate()

    def set_row_annotations_enabled(self, enabled, commit=True):
        if self._row_annotations_enabled != enabled:
            self._row_annotations_enabled = enabled
            self.row_annotations_cb.setChecked(enabled)
            self._data_loader.row_annotations_enabled = enabled
            if commit:
                self._invalidate()

    def set_col_annotations_enabled(self, enabled, commit=True):
        if self._col_annotations_enabled != enabled:
            self._col_annotations_enabled = enabled
            self.col_annotations_cb.setChecked(enabled)
            self._data_loader.col_annotations_enabled = enabled
            if commit:
                self._invalidate()

    def set_current_path(self, path):
        if samepath(self._current_path, path):
            return

        model = self.recent_model
        index = -1
        pathitem = None
        for i in range(model.rowCount()):
            item = model.item(i)
            data = item.data(Qt.UserRole) if item is not None else None
            if isinstance(data, RecentPath) and samepath(path, data.abspath):
                index, pathitem = i, data
                break

        rpaths = []

        if pathitem is None:
            assert index == -1
            pathitem = RecentPath.create(path, rpaths)

        if index != -1:
            item = model.takeRow(index)
        else:
            item = RecentPath_asqstandarditem(pathitem)

        model.insertRow(0, item)
        self._current_path = path
        self.recent_combo.setCurrentIndex(0)

        self._data_loader = get_data_loader(path)
        self.__update_summary()
        self.setup_gui()
        self._invalidate()

    def setup_gui(self):
        """ Use loader predefined values. If the value is None, set
        loader's parameter to widget's setting value.
        """
        loader = self._data_loader
        if loader.header_rows_count is not None:
            self.set_header_rows_count(loader.header_rows_count, False)
        else:
            loader.header_rows_count = self._header_rows_count

        if loader.header_cols_count is not None:
            self.set_header_cols_count(loader.header_cols_count, False)
        else:
            loader.header_cols_count = self._header_cols_count

        if loader.transposed is not None:
            self._cells_in_rows = not loader.transposed
        else:
            loader.transposed = not self._cells_in_rows

        if loader.sample_rows_enabled is not None:
            self.set_sample_rows_enabled(loader.sample_rows_enabled, False)
        else:
            loader.sample_rows_enabled = self._sample_rows_enabled

        if loader.sample_cols_enabled is not None:
            self.set_sample_cols_enabled(loader.sample_cols_enabled, False)
        else:
            loader.sample_cols_enabled = self._sample_cols_enabled

        if loader.sample_rows_p is not None:
            self.set_sample_rows_p(loader.sample_rows_p, False)
        else:
            loader.sample_rows_p = self._sample_rows_p

        if loader.sample_cols_p is not None:
            self.set_sample_cols_p(loader.sample_cols_p, False)
        else:
            loader.sample_cols_p = self._sample_cols_p

        if loader.row_annotation_file is not None:
            index = insert_recent_path(
                self.row_annotations_combo.model(),
                self.resolve_path(loader.row_annotation_file)
            )
            self.row_annotations_combo.setCurrentIndex(index)
            self.set_row_annotations_enabled(
                loader.row_annotations_enabled, False
            )
        else:
            self.row_annotations_combo.setCurrentIndex(-1)
            self.set_row_annotations_enabled(False, False)

        if loader.col_annotation_file is not None:
            index = insert_recent_path(
                self.col_annotations_combo.model(),
                self.resolve_path(loader.col_annotation_file)
            )
            self.col_annotations_combo.setCurrentIndex(index)
            self.set_col_annotations_enabled(
                loader.col_annotations_enabled, False)
        else:
            self.col_annotations_combo.setCurrentIndex(-1)
            self.set_col_annotations_enabled(False, False)

        self.header_rows_spin.setEnabled(loader.FIXED_FORMAT)
        self.header_cols_spin.setEnabled(loader.FIXED_FORMAT)
        self.data_struct_box.setEnabled(loader.FIXED_FORMAT)

        self.annotation_files_box.setEnabled(loader.ENABLE_ANNOTATIONS)

    def __update_summary(self):
        size = self._data_loader.file_size
        ncols = self._data_loader.n_cols
        nrows = self._data_loader.n_rows
        text = []
        if size is not None:
            text += [sizeformat(size)]
        if nrows is not None:
            text += ["{:n} rows".format(nrows)]
        if nrows is not None:
            text += ["{:n} columns".format(ncols)]

        self.summary_label.setText(", ".join(text))

    def current_path(self):
        return self._current_path

    def _select_recent(self, index):
        # type: (int) -> None
        # select a file from the recent list (entered via combo box `activate`)
        assert 0 <= index < self.recent_model.rowCount()
        item = self.recent_model.item(index)
        pathitem = item.data(Qt.UserRole)
        assert isinstance(pathitem, RecentPath)
        self.set_current_path(pathitem.abspath)

    @Slot()
    def browse(self):
        dlg = QFileDialog(self)
        dlg.setAcceptMode(QFileDialog.AcceptOpen)
        dlg.setFileMode(QFileDialog.ExistingFile)

        filters = Formats
        dlg.setNameFilters(filters)
        if filters:
            dlg.selectNameFilter(filters[0])
        if dlg.exec_() == QFileDialog.Accepted:
            filename = dlg.selectedFiles()[0]
            self.set_current_path(filename)

    @Slot()
    def browse_row_annotations(self):
        dlg = QFileDialog(
            self, acceptMode=QFileDialog.AcceptOpen,
            fileMode=QFileDialog.ExistingFile
        )
        filters = AnnotationFormats
        dlg.setNameFilters(filters)

        if filters:
            dlg.selectNameFilter(filters[0])
        if dlg.exec_() == QFileDialog.Accepted:
            filename = dlg.selectedFiles()[0]
            m = self.row_annotations_combo.model()  # type: QStandardItemModel
            pathitem = RecentPath.create(filename, [])
            index = insert_recent_path(m, pathitem)
            self.row_annotations_combo.setCurrentIndex(index)
            self._invalidate()

    @Slot()
    def browse_col_annotations(self):
        dlg = QFileDialog(
            self, acceptMode=QFileDialog.AcceptOpen,
            fileMode=QFileDialog.ExistingFile
        )
        filters = AnnotationFormats
        dlg.setNameFilters(filters)

        if filters:
            dlg.selectNameFilter(filters[0])
        if dlg.exec_() == QFileDialog.Accepted:
            filename = dlg.selectedFiles()[0]
            m = self.col_annotations_combo.model()  # type: QStandardItemModel
            pathitem = RecentPath.create(filename, [])
            index = insert_recent_path(m, pathitem)
            self.col_annotations_combo.setCurrentIndex(index)
            self._invalidate()

    def _invalidate(self):
        self.set_modified(True)

    def set_modified(self, modified):
        if modified:
            text = "Load data"
        else:
            text = "Reload"

        self.load_data_button.setText(text)
        self.load_data_button.setAutoDefault(modified)
        # Setting autoDefault once also sets default which persists even after
        # settings autoDefault back to False??
        self.load_data_button.setDefault(modified)
        self.Information.modified(shown=modified)

    def commit(self):
        path = self._current_path
        if not path:
            return
        self.Outputs.data.send(self._data_loader())
        self.show_error_messages()
        self.set_modified(False)

    def show_error_messages(self):
        self.Error.row_annotation_mismatch.clear()
        self.Error.col_annotation_mismatch.clear()
        self.Error.inadequate_headers.clear()
        errors = self._data_loader.errors
        if len(errors["row_annot_mismatch"]):
            self.Error.row_annotation_mismatch(*errors["row_annot_mismatch"])
        if len(errors["col_annot_mismatch"]):
            self.Error.col_annotation_mismatch(*errors["col_annot_mismatch"])
        if len(errors["inadequate_headers"]):
            self.Error.inadequate_headers(*errors["inadequate_headers"])
        if len(errors["reading_error"]):
            self.Error.reading_error()

    def onDeleteWidget(self):
        super().onDeleteWidget()

    def _saveState(self):
        maxitems = 15

        def dataiter(model, role=Qt.UserRole):
            return (model.data(model.index(i, 0), role)
                    for i in range(model.rowCount()))

        def recent_paths(model):
            return [self.relocate_path(el) for el in dataiter(model)
                    if isinstance(el, RecentPath)][:maxitems]

        self._recent = recent_paths(self.recent_model)
        self._recent_row_annotations = recent_paths(
            self.row_annotations_combo.model())
        self._recent_col_annotations = recent_paths(
            self.col_annotations_combo.model())
        self._last_path = self._current_path

    def relocate_path(self, path):
        basedir = self.workflowEnv().get("basedir", None)
        if not basedir or not path:
            return path
        return RecentPath.create(path.abspath, [("basedir", basedir)])

    def saveSettings(self):
        self._saveState()
        super().saveSettings()


def main(argv=None):
    app = QApplication(argv or [])
    w = OWLoadData()
    w.show()
    w.raise_()
    app.exec_()
    w.saveSettings()
    w.onDeleteWidget()


if __name__ == "__main__":
    sys.exit(main(sys.argv))



#!/usr/bin/env python3
import setuptools


if __name__ == '__main__':
    setuptools.setup(
        use_scm_version=True,
        setup_requires=[
            'setuptools-scm',
            'setuptools',
        ],
        install_requires=[
            'Orange3>=3.23.0',
            'orange3-bioinformatics>=4.0.0',
            'fastdtw==0.3.2',
            'pandas>=0.23',
            'loompy>=2.0.10',
            'xlrd~=1.2.0',
            'anndata>=0.6.21',
            'numpy',
            'scikit-learn',
        ],
        extras_require={
            'doc': ['sphinx', 'recommonmark'],
            'package': ['twine', 'wheel'],
            'test': [
                'coverage',
            ],
        },
    )



import unittest
import numpy as np
from orangecontrib.single_cell.preprocess.cca import MultiCCA, SVDCCA
from scipy.stats import pearsonr


class TestSVDCCA(unittest.TestCase):

    def test_random_data(self):
        """ Assert approximately decreasing explained variance of components."""
        np.random.seed(42)
        n1 = 200
        n2 = 300
        m = 100
        k = 20
        X = np.random.rand(n1, m)
        Y = np.random.rand(n2, m)
        model = SVDCCA(n_components=k, random_state=42)
        U, V = model.fit_transform(X, Y)
        slope, pvalue = pearsonr(model.correlations, np.arange(k))
        assert slope < 0
        assert pvalue < 0.05
        assert np.linalg.norm(U.T.dot(U) - np.eye(k)) < 1e-5
        assert np.linalg.norm(V.T.dot(V) - np.eye(k)) < 1e-5


class TestMultiCCA(unittest.TestCase):

    def test_random_data(self):
        """ Test approximately decreasing explained variance of components."""
        np.random.seed(42)
        ns = [10, 12, 14]
        m = 100
        k = 10
        Xs = [np.random.rand(n, m) for n in ns]
        model = MultiCCA(n_components=k, random_state=42)
        Ws = model.fit_transform(Xs)
        slope, pvalue = pearsonr(model.correlations, np.arange(k))
        assert slope < 0
        assert pvalue < 0.05
        for W in Ws:
            # Normal but not orthogonal
            assert np.linalg.norm(np.linalg.norm(W, axis=0) - np.ones((k,))) < 1e-5

if __name__ == "__main__":
    unittest.main()



import sys
import pkg_resources
import warnings

import numpy as np

from AnyQt.QtCore import Signal
from AnyQt.QtWidgets import (
    QApplication, QVBoxLayout, QHBoxLayout, QFormLayout, QSpinBox, QComboBox,
    QButtonGroup, QLabel, QDoubleSpinBox, QGroupBox, QCheckBox, QRadioButton
)

from Orange.data import DiscreteVariable
from Orange.preprocess.preprocess import PreprocessorList
import Orange.widgets.data.owpreprocess
from Orange.widgets.data.owpreprocess import (
    PreprocessAction, Description, index_to_enum, enum_to_index
)
from Orange.widgets.data.utils.preprocess import (
    ParametersRole, DescriptionRole, Controller, BaseEditor
)
from Orange.widgets.settings import (
    DomainContextHandler, ContextSetting, Setting
)
from Orange.widgets.utils.itemmodels import DomainModel
from Orange.widgets.utils.sql import check_sql_input
from Orange.widgets.widget import Input, Output, Msg

from orangecontrib.single_cell.preprocess.scpreprocess import (
    LogarithmicScale, Binarize, Normalize, NormalizeSamples, Standardize,
    SelectMostVariableGenes, NormalizeGroups, DropoutGeneSelection,
    DropoutWarning
)


def icon_path(basename):
    return pkg_resources.resource_filename(__name__, "icons/" + basename)


class ScBaseEditor(BaseEditor):
    def __init__(self, parent=None, master=None, **kwargs):
        super().__init__(parent, **kwargs)


class LogarithmicScaleEditor(ScBaseEditor):
    DEFAULT_BASE = LogarithmicScale.BinaryLog

    def __init__(self, parent=None, **kwargs):
        super().__init__(parent, **kwargs)
        self.setLayout(QVBoxLayout())

        form = QFormLayout()
        self.base_cb = QComboBox()
        self.base_cb.addItems(["2 (Binary Logarithm)",
                               "e (Natural Logarithm)",
                               "10 (Common Logarithm)"])
        self.base_cb.currentIndexChanged.connect(self.changed)
        self.base_cb.activated.connect(self.edited)

        form.addRow("Logarithm Base:", self.base_cb)
        self.layout().addLayout(form)

    def setParameters(self, params):
        base = params.get("base", self.DEFAULT_BASE)
        self.base_cb.setCurrentIndex(
            enum_to_index(LogarithmicScale.Base, base))

    def parameters(self):
        return {"base": index_to_enum(LogarithmicScale.Base,
                                      self.base_cb.currentIndex())}

    @staticmethod
    def createinstance(params):
        base = params.get("base", LogarithmicScaleEditor.DEFAULT_BASE)
        return LogarithmicScale(base)

    def __repr__(self):
        return "Base: {}".format(self.base_cb.currentText())


class BinarizeEditor(ScBaseEditor):
    DEFAULT_CONDITION = Binarize.GreaterOrEqual
    DEFAULT_THRESHOLD = 1

    def __init__(self, parent=None, **kwargs):
        super().__init__(parent, **kwargs)
        self._threshold = self.DEFAULT_THRESHOLD

        self.setLayout(QVBoxLayout())
        form = QFormLayout()
        self.cond_cb = QComboBox()
        self.cond_cb.addItems(["Greater or Equal", "Greater"])
        self.cond_cb.currentIndexChanged.connect(self.changed)
        self.cond_cb.activated.connect(self.edited)

        self.thr_spin = QDoubleSpinBox(
            minimum=0, singleStep=0.5, decimals=1, value=self._threshold
        )
        self.thr_spin.valueChanged[float].connect(self._set_threshold)
        self.thr_spin.editingFinished.connect(self.edited)

        form.addRow("Condition:", self.cond_cb)
        form.addRow("Threshold:", self.thr_spin)
        self.layout().addLayout(form)

    def _set_threshold(self, t):
        if self._threshold != t:
            self._threshold = t
            self.thr_spin.setValue(t)
            self.changed.emit()

    def setParameters(self, params):
        cond = params.get("condition", self.DEFAULT_CONDITION)
        self.cond_cb.setCurrentIndex(enum_to_index(Binarize.Condition, cond))
        self._set_threshold(params.get("threshold", self.DEFAULT_THRESHOLD))

    def parameters(self):
        cond = index_to_enum(Binarize.Condition, self.cond_cb.currentIndex())
        return {"condition": cond, "threshold": self._threshold}

    @staticmethod
    def createinstance(params):
        condition = params.get("condition", BinarizeEditor.DEFAULT_CONDITION)
        threshold = params.get("threshold", BinarizeEditor.DEFAULT_THRESHOLD)
        return Binarize(condition, threshold)

    def __repr__(self):
        return "Condition: {}, Threshold: {}".format(
            self.cond_cb.currentText(), self.thr_spin.value()
        )


class NormalizeEditor(ScBaseEditor):
    DEFAULT_GROUP_BY = False
    DEFAULT_GROUP_VAR = None
    DEFAULT_METHOD = Normalize.CPM

    def __init__(self, parent=None, master=None, **kwargs):
        super().__init__(parent, **kwargs)
        self._group_var = self.DEFAULT_GROUP_VAR
        self._master = master
        self._master.input_data_changed.connect(self._set_model)
        self.setLayout(QVBoxLayout())

        form = QFormLayout()
        cpm_b = QRadioButton("Counts per million", checked=True)
        med_b = QRadioButton("Median")
        self.group = QButtonGroup()
        self.group.buttonClicked.connect(self._on_button_clicked)
        for i, button in enumerate([cpm_b, med_b]):
            index = index_to_enum(Normalize.Method, i).value
            self.group.addButton(button, index - 1)
            form.addRow(button)

        self.group_by_check = QCheckBox("Cell Groups: ",
                                        enabled=self.DEFAULT_GROUP_BY)
        self.group_by_check.clicked.connect(self.edited)
        self.group_by_combo = QComboBox(enabled=self.DEFAULT_GROUP_BY)
        self.group_by_model = DomainModel(
            order=(DomainModel.METAS, DomainModel.CLASSES),
            valid_types=DiscreteVariable,
            alphabetical=True
        )
        self.group_by_combo.setModel(self.group_by_model)
        self.group_by_combo.currentIndexChanged.connect(self.changed)
        self.group_by_combo.activated.connect(self.edited)

        form.addRow(self.group_by_check, self.group_by_combo)
        self.layout().addLayout(form)

        self._set_model()

    def _set_model(self):
        data = self._master.data
        self.group_by_model.set_domain(data and data.domain)
        enable = bool(self.group_by_model)
        self.group_by_check.setChecked(False)
        self.group_by_check.setEnabled(enable)
        self.group_by_combo.setEnabled(enable)
        if self.group_by_model:
            self.group_by_combo.setCurrentIndex(0)
            if self._group_var and self._group_var in data.domain:
                index = self.group_by_model.indexOf(self._group_var)
                self.group_by_combo.setCurrentIndex(index)
        else:
            self.group_by_combo.setCurrentText(None)

    def _on_button_clicked(self):
        self.changed.emit()
        self.edited.emit()

    def setParameters(self, params):
        method = params.get("method", self.DEFAULT_METHOD)
        index = enum_to_index(Normalize.Method, method)
        self.group.buttons()[index].setChecked(True)
        self._group_var = params.get("group_var", self.DEFAULT_GROUP_VAR)
        group = bool(self._group_var and self.group_by_model)
        if group:
            index = self.group_by_model.indexOf(self._group_var)
            self.group_by_combo.setCurrentIndex(index)
        group_by = params.get("group_by", self.DEFAULT_GROUP_BY)
        self.group_by_check.setChecked(group_by and group)

    def parameters(self):
        index = self.group_by_combo.currentIndex()
        group_var = self.group_by_model[index] if index > -1 else None
        group_by = self.group_by_check.isChecked()
        method = index_to_enum(Normalize.Method, self.group.checkedId())
        return {"group_var": group_var, "group_by": group_by, "method": method}

    @staticmethod
    def createinstance(params):
        group_var = params.get("group_var")
        group_by = params.get("group_by", NormalizeEditor.DEFAULT_GROUP_BY)
        method = params.get("method", NormalizeEditor.DEFAULT_METHOD)
        return NormalizeGroups(group_var, method) \
            if group_by and group_var else NormalizeSamples(method)

    def __repr__(self):
        method = self.group.button(self.group.checkedId()).text()
        index = self.group_by_combo.currentIndex()
        group_var = self.group_by_model[index] if index > -1 else None
        group_by = self.group_by_check.isChecked()
        group_text = ", Grouped by: {}".format(group_var) if group_by else ""
        return "Method: {}".format(method) + group_text


class StandardizeEditor(ScBaseEditor):
    DEFAULT_LOWER_CLIP = False
    DEFAULT_UPPER_CLIP = False
    DEFAULT_LOWER_BOUND = -10
    DEFAULT_UPPER_BOUND = 10

    def __init__(self, parent=None, **kwargs):
        super().__init__(parent, **kwargs)
        self._lower_bound = self.DEFAULT_LOWER_BOUND
        self._upper_bound = self.DEFAULT_UPPER_BOUND

        self.setLayout(QVBoxLayout())

        box = QGroupBox(title="Clipping", flat=True)
        form = QFormLayout()
        self.lower_check = QCheckBox("Lower Bound: ")
        self.lower_check.clicked.connect(self.edited)
        self.lower_spin = QSpinBox(
            minimum=-99, maximum=0, value=self._lower_bound
        )
        self.lower_spin.valueChanged[int].connect(self._set_lower_bound)
        self.lower_spin.editingFinished.connect(self.edited)

        self.upper_check = QCheckBox("Upper Bound: ")
        self.upper_check.clicked.connect(self.edited)
        self.upper_spin = QSpinBox(value=self._upper_bound)
        self.upper_spin.valueChanged[int].connect(self._set_upper_bound)
        self.upper_spin.editingFinished.connect(self.edited)

        form.addRow(self.lower_check, self.lower_spin)
        form.addRow(self.upper_check, self.upper_spin)
        box.setLayout(form)
        self.layout().addWidget(box)

    def _set_lower_bound(self, x):
        if self._lower_bound != x:
            self._lower_bound = x
            self.lower_spin.setValue(x)
            self.changed.emit()

    def _set_upper_bound(self, x):
        if self._upper_bound != x:
            self._upper_bound = x
            self.upper_spin.setValue(x)
            self.changed.emit()

    def setParameters(self, params):
        lower_clip = params.get("lower_clip", self.DEFAULT_LOWER_CLIP)
        self.lower_check.setChecked(lower_clip)
        self._set_lower_bound(params.get("lower", self.DEFAULT_LOWER_BOUND))
        upper_clip = params.get("upper_clip", self.DEFAULT_UPPER_CLIP)
        self.upper_check.setChecked(upper_clip)
        self._set_upper_bound(params.get("upper", self.DEFAULT_UPPER_BOUND))

    def parameters(self):
        return {"lower_clip": self.lower_check.isChecked(),
                "lower": self._lower_bound,
                "upper_clip": self.upper_check.isChecked(),
                "upper": self._upper_bound}

    @staticmethod
    def createinstance(params):
        lower, upper = None, None
        if params.get("lower_clip", StandardizeEditor.DEFAULT_LOWER_CLIP):
            lower = params.get("lower", StandardizeEditor.DEFAULT_LOWER_BOUND)
        if params.get("upper_clip", StandardizeEditor.DEFAULT_UPPER_CLIP):
            upper = params.get("upper", StandardizeEditor.DEFAULT_UPPER_BOUND)
        return Standardize(lower, upper)

    def __repr__(self):
        clips = []
        if self.lower_check.isChecked():
            clips.append("Lower Bound: {}".format(self.lower_spin.value()))
        if self.upper_check.isChecked():
            clips.append("Upper Bound: {}".format(self.upper_spin.value()))
        return ", ".join(clips) if clips else "No Clipping"


class SelectGenesEditor(ScBaseEditor):
    DEFAULT_N_GENS = 1000
    DEFAULT_METHOD = SelectMostVariableGenes.Dispersion
    DEFAULT_COMPUTE_STATS = True
    DEFAULT_N_GROUPS = 20

    def __init__(self, parent=None, **kwargs):
        super().__init__(parent, **kwargs)
        self.setLayout(QVBoxLayout())
        self._n_genes = self.DEFAULT_N_GENS
        self._n_groups = self.DEFAULT_N_GROUPS

        form = QFormLayout()
        self.n_genes_spin = QSpinBox(minimum=1, maximum=10 ** 6,
                                     value=self._n_genes)
        self.n_genes_spin.valueChanged[int].connect(self._set_n_genes)
        self.n_genes_spin.editingFinished.connect(self.edited)
        form.addRow("Number of genes:", self.n_genes_spin)
        self.layout().addLayout(form)

        disp_b = QRadioButton("Dispersion", checked=True)
        vari_b = QRadioButton("Variance")
        mean_b = QRadioButton("Mean")
        self.group = QButtonGroup()
        self.group.buttonClicked.connect(self._on_button_clicked)
        for i, button in enumerate([disp_b, vari_b, mean_b]):
            index = index_to_enum(SelectMostVariableGenes.Method, i).value
            self.group.addButton(button, index - 1)
            form.addRow(button)

        self.stats_check = QCheckBox("Compute statistics for",
                                     checked=self.DEFAULT_COMPUTE_STATS)
        self.stats_check.clicked.connect(self.edited)
        self.n_groups_spin = QSpinBox(minimum=1, value=self._n_groups)
        self.n_groups_spin.valueChanged[int].connect(self._set_n_groups)
        self.n_groups_spin.editingFinished.connect(self.edited)

        box = QHBoxLayout()
        box.addWidget(self.stats_check)
        box.addWidget(self.n_groups_spin)
        box.addWidget(QLabel("gene groups."))
        box.addStretch()
        self.layout().addLayout(box)

    def _set_n_genes(self, n):
        if self._n_genes != n:
            self._n_genes = n
            self.n_genes_spin.setValue(n)
            self.changed.emit()

    def _set_n_groups(self, n):
        if self._n_groups != n:
            self._n_groups = n
            self.n_groups_spin.setValue(n)
            self.changed.emit()

    def _on_button_clicked(self):
        self.changed.emit()
        self.edited.emit()

    def setParameters(self, params):
        self._set_n_genes(params.get("n_genes", self.DEFAULT_N_GENS))
        method = params.get("method", self.DEFAULT_METHOD)
        index = enum_to_index(SelectMostVariableGenes.Method, method)
        self.group.buttons()[index].setChecked(True)
        compute_stats = params.get("compute_stats", self.DEFAULT_COMPUTE_STATS)
        self.stats_check.setChecked(compute_stats)
        self._set_n_groups(params.get("n_groups", self.DEFAULT_N_GROUPS))

    def parameters(self):
        method = index_to_enum(SelectMostVariableGenes.Method,
                               self.group.checkedId())
        return {"n_genes": self._n_genes, "method": method,
                "compute_stats": self.stats_check.isChecked(),
                "n_groups": self._n_groups}

    @staticmethod
    def createinstance(params):
        method = params.get("method", SelectGenesEditor.DEFAULT_METHOD)
        n_genes = params.get("n_genes", SelectGenesEditor.DEFAULT_N_GENS)
        compute_stats = params.get(
            "compute_stats", SelectGenesEditor.DEFAULT_COMPUTE_STATS)
        n_groups = params.get("n_groups", SelectGenesEditor.DEFAULT_N_GROUPS) \
            if compute_stats else None
        return SelectMostVariableGenes(method, n_genes, n_groups)

    def __repr__(self):
        method = self.group.button(self.group.checkedId()).text()
        text = "Method: {}, Number of Genes: {}".format(method, self._n_genes)
        if self.stats_check.isChecked():
            text += ", Number of Groups: {}".format(self._n_groups)
        return text


class DropoutEditor(ScBaseEditor):
    DEFAULT_N_GENES = 1000

    def __init__(self, parent=None, **kwargs):
        super().__init__(parent, **kwargs)
        self.setLayout(QVBoxLayout())
        self._n_genes = self.DEFAULT_N_GENES

        form = QFormLayout()
        self.n_genes_spin = QSpinBox(minimum=1, maximum=10 ** 6,
                                     value=self._n_genes)
        self.n_genes_spin.valueChanged[int].connect(self._set_n_genes)
        self.n_genes_spin.editingFinished.connect(self.edited)
        form.addRow("Number of genes:", self.n_genes_spin)
        self.layout().addLayout(form)

    def _set_n_genes(self, n):
        if self._n_genes != n:
            self._n_genes = n
            self.n_genes_spin.setValue(n)
            self.changed.emit()

    def setParameters(self, params):
        self._set_n_genes(params.get("n_genes", self.DEFAULT_N_GENES))

    def parameters(self):
        return {"n_genes": self._n_genes}

    @staticmethod
    def createinstance(params):
        n_genes = params.get("n_genes", DropoutEditor.DEFAULT_N_GENES)
        return DropoutGeneSelection(n_genes)

    def __repr__(self):
        return "Number of Genes: {}".format(self._n_genes)


PREPROCESS_ACTIONS = [
    PreprocessAction(
        "Logarithmic Scale", "preprocess.log_scale", "Value-Based",
        Description("Logarithmic Scale",
                    icon_path("LogarithmicScale.svg")),
        LogarithmicScaleEditor
    ),
    PreprocessAction(
        "Binarize Expression", "preprocess.binarize", "Value-Based",
        Description("Binarize Expression",
                    icon_path("Binarize.svg")),
        BinarizeEditor
    ),
    PreprocessAction(
        "Normalize Samples", "preprocess.normalize", "Row-Based",
        Description("Normalize Samples",
                    icon_path("Normalize.svg")),
        NormalizeEditor
    ),
    PreprocessAction(
        "Standardize Genes", "preprocess.standardize", "Column-Based",
        Description("Standardize Genes",
                    icon_path("Standardize.svg")),
        StandardizeEditor
    ),
    PreprocessAction(
        "Select Most Variable Genes", "preprocess.select_genes",
        "Column-Based",
        Description("Select Most Variable Genes",
                    icon_path("SelectGenes.svg")),
        SelectGenesEditor
    ),
    PreprocessAction(
        "Dropout Gene Selection", "preprocess.dropout", "Column-Based",
        Description("Dropout Gene Selection", icon_path("Dropout.svg")),
        DropoutEditor
    )
]


class ScController(Controller):
    def __init__(self, view, model=None, parent=None):
        super().__init__(view, model, parent)
        self._master = parent

    def createWidgetFor(self, index):
        definition = index.data(DescriptionRole)
        widget = definition.viewclass(master=self._master)
        return widget


class OWscPreprocess(Orange.widgets.data.owpreprocess.OWPreprocess):
    name = "Single Cell Preprocess"
    description = "Preprocess Single Cell data set"
    icon = "icons/SingleCellPreprocess.svg"
    priority = 220

    class Inputs:
        data = Input("Data", Orange.data.Table)

    class Outputs:
        preprocessed_data = Output("Preprocessed Data", Orange.data.Table)

    class Error(Orange.widgets.data.owpreprocess.OWPreprocess.Error):
        unknown_error = Msg("{}")
        discrete_attributes = Msg("Data with discrete attributes "
                                  "can not be preprocessed.")

    class Warning(Orange.widgets.data.owpreprocess.OWPreprocess.Warning):
        missing_values = Msg("Missing values have been replaced with 0.")
        dropout_warning = Msg("{}")
        bad_pp_combination = Msg("Dropout should not be used after "
                                 "Normalization/Log-transformation.")

    PREPROCESSORS = PREPROCESS_ACTIONS
    DEFAULT_PP = {"preprocessors": [("preprocess.normalize", {}),
                                    ("preprocess.log_scale", {}),
                                    ("preprocess.select_genes", {}),
                                    ("preprocess.standardize", {})]}
    CONTROLLER = ScController
    storedsettings = Setting(DEFAULT_PP)
    group_var = ContextSetting(None)
    settingsHandler = DomainContextHandler()

    input_data_changed = Signal()

    @Inputs.data
    @check_sql_input
    def set_data(self, data=None):
        """Set the input dataset."""
        self.closeContext()
        self.data = data
        self.openContext(data)
        self.check_data()
        self.input_data_changed.emit()
        self.load_group_var()

    def check_data(self):
        self.Error.discrete_attributes.clear()
        if self.data and self.data.domain.has_discrete_attributes():
            self.data = None
            self.Error.discrete_attributes()

        self.Warning.missing_values.clear()
        if self.data and np.isnan(self.data.X).any():
            self.data.X = np.nan_to_num(self.data.X)
            self.Warning.missing_values()

    def load_group_var(self):
        for index in range(self.preprocessormodel.rowCount()):
            item = self.preprocessormodel.item(index)
            params = item.data(ParametersRole)
            if "group_var" in params:
                params["group_var"] = self.group_var
                item.setData(params, ParametersRole)

    def save(self, model):
        d = {"name": ""}
        preprocessors = []
        for i in range(model.rowCount()):
            item = model.item(i)
            pp_def = item.data(DescriptionRole)
            params = item.data(ParametersRole)
            group_var = params.get("group_var")
            if group_var is not None:
                self.group_var = group_var
                params = dict(params)
                params["group_var"] = None
            preprocessors.append((pp_def.qualname, params))

        d["preprocessors"] = preprocessors
        return d

    def apply(self):
        self.storeSpecificSettings()
        preprocessor = self.buildpreproc()
        data = None
        self.Warning.dropout_warning.clear()
        self.Warning.bad_pp_combination.clear()
        self.Error.unknown_error.clear()
        if self.data is not None:
            try:
                with warnings.catch_warnings(record=True) as w:
                    data = preprocessor(self.data)
                for warning in w:
                    if issubclass(warning.category, DropoutWarning):
                        self.Warning.dropout_warning(warning.message)
                    else:
                        warnings.warn(warning.message)
                if self.is_bad_combination(preprocessor):
                    self.Warning.bad_pp_combination()
            except (ValueError, ZeroDivisionError) as e:
                self.Error.unknown_error(str(e))
        self.Outputs.preprocessed_data.send(data)

    @staticmethod
    def is_bad_combination(pp) -> bool:
        if not isinstance(pp, PreprocessorList):
            return False

        index_norm, index_log, index_dropout = None, None, None
        for i in range(len(pp.preprocessors)):
            if isinstance(pp.preprocessors[i], NormalizeSamples):
                index_norm = i
            elif isinstance(pp.preprocessors[i], LogarithmicScale):
                index_log = i
            elif isinstance(pp.preprocessors[i], DropoutGeneSelection):
                index_dropout = i
        return index_dropout is not None and \
               (index_norm is not None and index_dropout > index_norm or
                index_log is not None and index_dropout > index_log)


def main(args=None):
    from Orange.data import Table

    app = QApplication(args or [])
    w = OWscPreprocess()
    w.set_data(Table("iris"))
    w.show()
    w.raise_()
    app.exec_()
    w.saveSettings()
    w.onDeleteWidget()


if __name__ == "__main__":
    sys.exit(main())



import numpy as np
from AnyQt.QtCore import Qt
from Orange.clustering import hierarchical
from Orange.clustering.hierarchical import leaves
from Orange.data import (DiscreteVariable, Table, Domain, ContinuousVariable)
from Orange.data.filter import Values, FilterDiscrete
from Orange.distance import Euclidean
from Orange.widgets import widget, gui
from Orange.widgets.settings import Setting, ContextSetting, DomainContextHandler
from Orange.widgets.utils.annotated_data import ANNOTATED_DATA_SIGNAL_NAME, create_annotated_table
from Orange.widgets.utils.itemmodels import DomainModel
from Orange.widgets.utils.signals import Input, Output
from Orange.widgets.utils.sql import check_sql_input
from Orange.preprocess import SklImpute
from Orange.widgets.widget import OWWidget, Msg

from orangecontrib.single_cell.preprocess.clusteranalysis import ClusterAnalysis
from orangecontrib.single_cell.widgets.contingency_table import ContingencyTable


class OWDotMatrix(widget.OWWidget):
    name = "Dot Matrix"
    description = "Perform cluster analysis."
    icon = "icons/DotMatrix.svg"
    priority = 410

    class Inputs:
        data = Input("Data", Table, default=True)

    class Outputs:
        selected_data = Output("Selected Data", Table, default=True)
        annotated_data = Output(ANNOTATED_DATA_SIGNAL_NAME, Table)
        contingency = Output("Contingency Table", Table)

    class Error(OWWidget.Error):
        no_discrete_variable = Msg("No discrete variables in data.")

    class Warning(OWWidget.Warning):
        to_many_attributes = Msg("Too many genes on input, first {} genes displayed.")

    GENE_MAXIMUM = 100
    CELL_SIZES = (14, 22, 30)
    AGGREGATE_F = [
        lambda x: np.mean(x, axis=0),
        lambda x: np.median(x, axis=0),
        lambda x: np.min(x, axis=0),
        lambda x: np.max(x, axis=0),
        lambda x: np.mean(x > 0, axis=0),
    ]
    AGGREGATE_NAME = [
        "Mean expression",
        "Median expression",
        "Min expression",
        "Max expression",
        "Fraction expressing"
    ]

    settingsHandler = DomainContextHandler()
    cluster_var = ContextSetting(None)
    aggregate_ix = ContextSetting(0)  # type: int
    biclustering = ContextSetting(True)
    transpose = ContextSetting(False)
    log_scale = ContextSetting(False)
    normalize = ContextSetting(True)
    cell_size_ix = ContextSetting(2)  # type: int
    selection_indices = ContextSetting(set())
    auto_apply = Setting(True)

    want_main_area = True

    def __init__(self):
        super().__init__()

        self.feature_model = DomainModel(valid_types=DiscreteVariable)
        self._init_vars()
        self._set_info_string()

        box = gui.vBox(self.controlArea, "Cluster Variable")
        gui.comboBox(box, self, "cluster_var", sendSelectedValue=True,
                     model=self.feature_model, callback=self._aggregate_data)

        box = gui.vBox(self.controlArea, "Aggregation")
        gui.comboBox(box, self, "aggregate_ix", sendSelectedValue=False,
                     items=self.AGGREGATE_NAME, callback=self._aggregate_data)

        box = gui.vBox(self.controlArea, "Options")
        gui.checkBox(box, self, "biclustering", "Order cells and genes",
                     callback=self._calculate_table_values)
        gui.checkBox(box, self, "transpose", "Transpose",
                     callback=self._calculate_table_values)
        gui.checkBox(box, self, "log_scale", "Log scale",
                     callback=self._calculate_table_values)
        gui.checkBox(box, self, "normalize", "Normalize data",
                     callback=self._calculate_table_values)

        box = gui.vBox(self.controlArea, "Plot Size")
        gui.radioButtons(box, self, "cell_size_ix", btnLabels=("S", "M", "L"),
                         callback=lambda: self.tableview.set_cell_size(self.CELL_SIZES[self.cell_size_ix]),
                         orientation=Qt.Horizontal)

        gui.rubber(self.controlArea)

        self.apply_button = gui.auto_commit(
            self.controlArea, self, "auto_apply", "&Apply", box=False)

        self.tableview = ContingencyTable(self)
        self.mainArea.layout().addWidget(self.tableview)

    def _init_vars(self):
        self.data = None  # type: Table
        self.matrix = None
        self.clusters = None
        self.clusters_unordered = None
        self.aggregated_data = None
        self.cluster_var = None
        self.selected_names = {}

    def _set_info_string(self):
        formatstr = "{} genes\n{} cells\n{} clusters"
        if self.data:
            self.info.set_input_summary(
                str(len(self.data)),
                formatstr.format(len(self.data.domain.attributes), len(self.data), len(self.clusters_unordered)))
        else:
            self.info.set_input_summary(self.info.NoInput)

    @Inputs.data
    @check_sql_input
    def set_data(self, data):
        if self.feature_model:
            self.closeContext()

        self._init_vars()
        self.Error.no_discrete_variable.clear()
        self.Warning.clear()
        self.data = data
        if self.data:
            self.feature_model.set_domain(self.data.domain)
            if self.feature_model:
                self.openContext(self.data)
                if self.cluster_var is None:
                    self.cluster_var = self.feature_model[0]
                self._aggregate_data()
            else:
                self.tableview.clear()
                self.Error.no_discrete_variable()
                self.data = None
                self._set_info_string()
                self.commit()
        else:
            self.tableview.clear()
            self._set_info_string()
            self.commit()

    @staticmethod
    def _group_by(table: Table, var: DiscreteVariable):
        column = table.get_column_view(var)[0]
        return (table[column == value] for value in np.unique(column))

    @staticmethod
    def _transpose(matrix: np.ndarray, clusters, genes):
        return matrix.T, np.array([str(g) for g in genes]), [ContinuousVariable(c) for c in clusters]

    @staticmethod
    def _normalize(matrix: np.ndarray):
        matrix = (matrix - np.mean(matrix, axis=0, keepdims=True)) / (
                np.std(matrix, axis=0, keepdims=True) + 1e-10)
        matrix[matrix < -3] = -3
        matrix[matrix > 3] = 3
        return matrix

    @staticmethod
    def _norm_min_max(matrix: np.ndarray):
        matrix = matrix - matrix.min()
        return matrix / (matrix.max() + 1e-12)

    def _aggregate_data(self):
        self.Warning.clear()
        if self.data is None:
            return

        self.clusters_unordered = np.array(
            [self.cluster_var.values[int(ix)]
             for ix in np.unique(self.data.get_column_view(self.cluster_var)[0])])
        self._set_info_string()

        if len(self.data.domain.attributes) > self.GENE_MAXIMUM:
            self.Warning.to_many_attributes(self.GENE_MAXIMUM)

        self.aggregated_data = np.stack([self.AGGREGATE_F[self.aggregate_ix](cluster.X[:, :self.GENE_MAXIMUM])
                                         for cluster in self._group_by(self.data, self.cluster_var)],
                                        axis=0)
        self._calculate_table_values()

    def _calculate_table_values(self):
        genes = self.data.domain.attributes[:self.GENE_MAXIMUM]
        matrix = self.aggregated_data
        clusters = self.clusters_unordered
        if self.transpose:
            matrix, clusters, genes = self._transpose(matrix, clusters, genes)

        # create data table since imputation of nan values is required
        matrix = Table(Domain(genes), matrix)
        matrix_before_norm = matrix.copy()  # for tooltip
        matrix = SklImpute()(matrix)

        if self.log_scale:
            matrix.X = np.log(matrix.X + 1)
        if self.normalize:
            matrix.X = self._normalize(matrix.X)

        # values must be in range [0, 1] for visualisation
        matrix.X = self._norm_min_max(matrix.X)

        if self.biclustering:
            cluster_order, gene_order = self.cluster_data(matrix)
        else:
            cluster_order, gene_order = np.arange(matrix.X.shape[0]), np.arange(matrix.X.shape[1])

        # reorder
        self.matrix = matrix[cluster_order][:, gene_order]
        self.matrix_before_norm = matrix_before_norm[cluster_order][:, gene_order]
        self.clusters = clusters[cluster_order]

        self._refresh_table()
        self._update_selection()
        self._invalidate()

    def cluster_data(self, matrix):
        with self.progressBar():
            # cluster rows
            if len(matrix) > 1:
                rows_distances = Euclidean(matrix)
                cluster = hierarchical.dist_matrix_clustering(rows_distances)
                row_order = hierarchical.optimal_leaf_ordering(
                    cluster, rows_distances, progress_callback=self.progressBarSet)
                row_order = np.array([x.value.index for x in leaves(row_order)])
            else:
                row_order = np.array([0])

            # cluster columns
            if matrix.X.shape[1] > 1:
                columns_distances = Euclidean(matrix, axis=0)
                cluster = hierarchical.dist_matrix_clustering(columns_distances)
                columns_order = hierarchical.optimal_leaf_ordering(
                    cluster, columns_distances,
                    progress_callback=self.progressBarSet)
                columns_order = np.array([x.value.index for x in leaves(columns_order)])
            else:
                columns_order = np.array([0])
        return row_order, columns_order

    def _refresh_table(self):
        if self.matrix is None:
            return

        columns = np.array([str(x) for x in self.matrix.domain.attributes])
        rows = self.clusters
        # row_order, column_order = self.gene_order, self.cluster_order
        self.tableview.set_headers(rows, columns, circles=True,
                                   cell_size=self.CELL_SIZES[self.cell_size_ix], bold_headers=False)
        if self.matrix.X.size > 0:
            matrix = self.matrix.X

            def tooltip(i, j):
                cluster, gene, value = rows[i], columns[j], self.matrix_before_norm[i, j]
                return "Cluster: {}\nGene: {}\n{}: {:.1f}".format(
                    cluster, gene, self.AGGREGATE_NAME[self.aggregate_ix], value)

            self.tableview.update_table(matrix, tooltip=tooltip)

    def commit(self):
        if self.data is None:
            self.Outputs.selected_data.send(None)
            self.Outputs.annotated_data.send(None)
            self.Outputs.contingency.send(None)
            return

        if len(self.selection_indices):
            cluster_ids = set()
            gene_ids = set()
            for (ir, ic) in self.selection_indices:
                if not self.transpose:
                    cluster_ids.add(ir)
                    gene_ids.add(ic)
                else:
                    cluster_ids.add(ic)
                    gene_ids.add(ir)

            columns = self.clusters if self.transpose else [str(x) for x in self.matrix.domain.attributes]
            rows = self.clusters if not self.transpose else [str(x) for x in self.matrix.domain.attributes]
            new_domain = Domain([self.data.domain[columns[i]] for i in gene_ids],
                                self.data.domain.class_vars,
                                self.data.domain.metas)
            selected_data = Values([FilterDiscrete(self.cluster_var, [rows[i]])
                                    for i in cluster_ids],
                                   conjunction=False)(self.data)
            selected_data = selected_data.transform(new_domain)
            annotated_data = create_annotated_table(self.data,
                                                    np.where(np.in1d(self.data.ids, selected_data.ids, True)))
        else:
            selected_data = None
            annotated_data = create_annotated_table(self.data, [])

        clusters_values = list(set(self.clusters))
        table = ClusterAnalysis.contingency_table(
            self.matrix,
            DiscreteVariable("Gene" if self.transpose else self.cluster_var.name, clusters_values),
            [str(x) for x in self.matrix.domain.attributes],
            [[clusters_values.index(c)] for c in self.clusters]
        )

        self.Outputs.selected_data.send(selected_data)
        self.Outputs.annotated_data.send(annotated_data)
        self.Outputs.contingency.send(table)

    def _update_selection(self):
        """
        This function updates widget selection in case when any item is selected.
        It updates selection when order has changed
        """
        rows = self.clusters.tolist()
        columns = [str(x) for x in self.matrix.domain.attributes]
        if self.transpose:
            new_selection = {(rows.index(g), columns.index(c)) for g, c in self.selected_names}
        else:
            new_selection = {(rows.index(c), columns.index(g)) for g, c in self.selected_names}
        self.tableview.set_selection(new_selection)

    def _invalidate(self):
        self.save_selection_names()
        self.commit()

    def save_selection_names(self):
        """
        With this method we save the names of selected genes-clusters pairs, since options changes
        the columns, rows orders and we want to keep the selection.
        """
        self.selection_indices = self.tableview.get_selection()
        genes = self.clusters if self.transpose else [str(x) for x in self.matrix.domain.attributes]
        clusters = self.clusters if not self.transpose else [str(x) for x in self.matrix.domain.attributes]

        if self.transpose:
            self.selected_names = {(genes[g], clusters[c]) for g, c in self.selection_indices}
        else:
            self.selected_names = {(genes[g], clusters[c]) for c, g in self.selection_indices}


def test():
    from AnyQt.QtWidgets import QApplication
    app = QApplication([])

    w = OWDotMatrix()
    data = Table("iris")
    w.set_data(data)
    w.handleNewSignals()
    w.show()
    app.exec_()


if __name__ == "__main__":
    test()



import unittest

import numpy as np

import Orange.data
from Orange.widgets.tests.base import WidgetTest

from orangecontrib.single_cell.widgets.owfilter import OWFilter, log1p, expm1


class TestOWFilterCells(WidgetTest):
    def setUp(self):
        self.widget = self.create_widget(OWFilter)

    def tearDown(self):
        self.widget.onDeleteWidget()
        self.widget.deleteLater()
        del self.widget

    def test(self):
        domain = Orange.data.Domain(
            [Orange.data.ContinuousVariable("A1"),
             Orange.data.ContinuousVariable("A2"),
             Orange.data.ContinuousVariable("A3")]
        )
        data = Orange.data.Table.from_list(
            domain,
            [[0, 0, 0],
             [0, 0, 1],
             [0, 1, 2],
             [1, 2, 3]]
        )
        self.send_signal(self.widget.Inputs.data, data)

        out = self.get_output(self.widget.Outputs.data)
        self.assertTrue(len(out) == len(data))

        self.widget.limit_lower = 2
        self.widget.commit()
        out = self.get_output(self.widget.Outputs.data)
        self.assertEqual(len(out), 2)
        np.testing.assert_array_equal(
            np.count_nonzero(out.X, axis=1),
            [2, 3]
        )
        self.widget.set_filter_type(OWFilter.Genes)
        self.widget.limit_lower = 2
        self.widget.commit()
        out = self.get_output(self.widget.Outputs.data)
        np.testing.assert_array_equal(
            np.count_nonzero(out.X, axis=0),
            [2, 3]
        )
        self.widget.set_filter_type(OWFilter.Data)
        self.widget.limit_lower = 2
        self.widget.commit()
        out = self.get_output(self.widget.Outputs.data)
        np.testing.assert_array_equal(
            out.X,
            [[0, 0, 0],
             [0, 0, 0],
             [0, 0, 2],
             [0, 2, 3]]
        )

        self.widget.limit_upper = 2
        self.widget.commit()
        out = self.get_output(self.widget.Outputs.data)
        np.testing.assert_array_equal(
            out.X,
            [[0, 0, 0],
             [0, 0, 0],
             [0, 0, 2],
             [0, 2, 0]]
        )
        self.widget.set_upper_limit_enabled(False)
        self.widget.commit()
        out = self.get_output(self.widget.Outputs.data)
        np.testing.assert_array_equal(
            out.X,
            [[0, 0, 0],
             [0, 0, 0],
             [0, 0, 2],
             [0, 2, 3]]
        )
        self.widget.set_lower_limit_enabled(False)
        self.widget.commit()
        out = self.get_output(self.widget.Outputs.data)
        np.testing.assert_array_equal(
            out.X,
            [[0, 0, 0],
             [0, 0, 1],
             [0, 1, 2],
             [1, 2, 3]]
        )
        self.widget.set_upper_limit_enabled(True)
        self.widget.commit()
        out = self.get_output(self.widget.Outputs.data)
        np.testing.assert_array_equal(
            out.X,
            [[0, 0, 0],
             [0, 0, 0],
             [0, 0, 2],
             [0, 2, 3]]
        )
        #  For paint code coverage
        self.widget.grab()

        self.send_signal(self.widget.Inputs.data, None)
        self.assertIsNone(self.get_output(self.widget.Outputs.data))

    def test_utils(self):
        # test log1p and its inverse
        x = 10 ** np.arange(9)
        np.testing.assert_allclose(log1p(x), np.log10(1 + x))
        np.testing.assert_allclose(expm1(log1p(x)), x)
        x = 1 / (10 ** np.arange(1, 7))
        np.testing.assert_allclose(log1p(x), np.log10(1 + x))
        np.testing.assert_allclose(expm1(log1p(x)), x)


if __name__ == "__main__":
    unittest.main()
